/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/*
 * Portions of the code come from:
 *
 * XMMS - wrapper round fourier transforms
 * WMDiscoTux - Beat Detection
 * Fine Spectrum Analyser - graphical display of spectrum
 *
 */
#include "audio_window.h"
#include "audio_common.h"
#include "audio_thread.h"

#include <math.h>

#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <string.h>

#include <errno.h>

#include <arpa/inet.h>


void audio_window_update(gpointer widget, gint source,  GdkInputCondition con) ;
void audio_window_accept(gpointer data, gint source,  GdkInputCondition con)  ;

/* HEIGHT can be modified at your pleasure */
#define HEIGHT 128

/* Linearity of the amplitude scale (0.5 for linear, keep in [0.1, 0.9]) */
#define d 0.33

/* Time factor of the band dinamics. 3 means that the coefficient of the
   last value is half of the current one's. (see source) */
#define tau 3

/* Factor used for the diffusion. 4 means that half of the height is
   added to the neighbouring bars */
#define dif 4


// samples per second
#define SAMPLES RATE / (WIDTH*2)

 
#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)


enum {
	BEATING,
	SLOW,
	SLOW_NOT_SIGNALED,
	TRANSITION	
};

gdouble scale, x00, y00;
GdkPixmap *bar_pixmap = NULL, *draw1_pixmap = NULL, *draw2_pixmap = NULL;
GdkGC *gc1 = NULL;

gint16 bar_heights[CHANNELS][WIDTH];
int listen_sd ;
int sd ;
void (*beat_callback)(audio_events event, void *data) = NULL;
void *callback_data = NULL;
int delay, sens ;
gint callback_tag = 0 ; 


GtkWidget *beat_button ;
GtkWidget *slow_button ;
GtkWidget *checkbutton2;


int audio_window_get_samples_per_second() {
	return SAMPLES ;	
}

/*
 * Init the draw areas.
 *
 */
GdkPixmap *init_draw_area (GtkWidget *drawarea) {
	GdkPixmap *draw_pixmap ;
	
	// have to to this before we can create pixmaps
	gtk_widget_realize(drawarea);

	// new pixmap for drawing
	draw_pixmap = gdk_pixmap_new(drawarea->window,WIDTH,HEIGHT,-1);
 
  // set the draw pixmap as the background
	gdk_window_set_back_pixmap(drawarea->window,draw_pixmap,0);
  
	return draw_pixmap ;
}

/*
 * Draw the bars pixmap. 
 */
GdkPixmap *init_bars(GdkDrawable *drawable ) {
	GdkPixmap *bars ;
	GdkColor color;
	int i ;
	
	// pixmap for bars
	bars = gdk_pixmap_new(drawable,25, HEIGHT, gdk_rgb_get_visual()->depth);
	
 	for(i = 0; i < HEIGHT / 2; i++) {
		color.red = 0xFFFF;
		color.green = ((i * 255) / (HEIGHT / 2)) << 8;
		color.blue = 0;
		
		gdk_color_alloc(gdk_colormap_get_system(),&color);
		gdk_gc_set_foreground(gc1,&color);
		gdk_draw_line(bars,gc1,0,i,24,i);
	}
	for(i = 0; i < HEIGHT / 2; i++) {
		color.red = (255 - ((i * 255) / (HEIGHT / 2))) <<8;
		color.green = 0xFFFF;
		color.blue = 0;
		
		gdk_color_alloc(gdk_colormap_get_system(),&color);
		gdk_gc_set_foreground(gc1,&color);
    	gdk_draw_line(bars,gc1,0,i + (HEIGHT / 2),24,i + (HEIGHT / 2));
	}
	return bars;
}


// sens changed
void on_sens_vscale_value_changed (GtkRange *range, gpointer user_data) {
	sens = gtk_range_get_value(range) ;

	
}

// delay changed
void on_delay_vscale_value_changed (GtkRange *range, gpointer user_data) {
	delay = SAMPLES * gtk_range_get_value(range) ;
}


/*
 * Draw the spectrum on the drawarea
 *
 */
void draw_func(GtkWidget *area, GdkPixmap *draw_pixmap, GdkPixmap *bar_pixmap, gint16 bar_heights[WIDTH]) {
	gint i;
	
	if(!area || ! draw_pixmap || !bar_pixmap)
		return;
	
	gdk_draw_rectangle(draw_pixmap, gc1, TRUE, 0, 0, WIDTH, HEIGHT);
	
	for(i = 0; i < WIDTH; i++)
		gdk_draw_pixmap(draw_pixmap, gc1, bar_pixmap, 0, HEIGHT-1-bar_heights[i], i, HEIGHT-1-bar_heights[i], 1, bar_heights[i]);

	gdk_window_clear(area->window);
}

/*
 * Work out bar heights and draw
 *
 */
void draw_spect(GtkWidget *audioWindow, gint16 data[CHANNELS][WIDTH]) {
	gint i,j ;
	gdouble y;
	GtkWidget *drawarea ;
	
	if(!audioWindow)
		return;
	
	drawarea = lookup_widget(audioWindow, "drawingarea1") ;
	
	for(j=0; j< CHANNELS; j++) {
		
		for (i = 0; i < WIDTH; i++) {
			y = (gdouble)data[j][i] * (i + 1); /* Compensating the energy */
			y = ( log(y - x00) * scale + y00 ); /* Logarithmic amplitude */
			y = ( (dif-2)*y + /* FIXME: conditionals should be rolled out of the loop */
				(i==0       ? y : bar_heights[j][i-1]) +
				(i==WIDTH-1 ? y : bar_heights[j][i+1])) / dif; /* Add some diffusion */
			y = ((tau-1)*bar_heights[j][i] + y) / tau; /* Add some dynamics */
			bar_heights[j][i] = (gint16)y;
		}
	

		
		if(j==0) {
			draw_func(lookup_widget(audioWindow, "drawingarea1"), draw1_pixmap, bar_pixmap, bar_heights[j]);
		} else {
			draw_func(lookup_widget(audioWindow, "drawingarea2"), draw2_pixmap, bar_pixmap, bar_heights[j]);
		}
	}
	return;

}

/*
 * Clear the spectrum areas
 *
 */
void clear_spec(GtkWidget *widget) {
	GtkWidget *area = lookup_widget(widget, "drawingarea1") ;

	// draw black rectange
	gdk_draw_rectangle(draw1_pixmap, gc1, TRUE, 0, 0, WIDTH, HEIGHT);	
	gdk_draw_rectangle(draw2_pixmap, gc1, TRUE, 0, 0, WIDTH, HEIGHT);

	// redraw areas
	gdk_window_clear(area->window);
	area =  lookup_widget(widget, "drawingarea2") ;
	gdk_window_clear(area->window);
	
}



// need to fix here
void on_enable_button_toggled (GtkCheckButton *but, gpointer user_data) {

	
	printf("enabled button toggled\n") ;\
	
	if( gtk_toggle_button_get_active((GtkToggleButton*) but) && ! callback_tag ) {
		// active and not connected -> connect
		printf("AW: runnning audio theead\n") ;
		
		audio_thread_run() ;
		printf("AW: done starting thread\n") ;
	} else if (gtk_toggle_button_get_active((GtkToggleButton*) but) && callback_tag) {
		// active and connected -> don't connect 
		gtk_toggle_button_set_active((GtkToggleButton*) but, FALSE );	
		
	} else if (! gtk_toggle_button_get_active((GtkToggleButton*) but) && callback_tag ) {
		// not active and connected -> disconnect
		printf("terminating audio thread\n") ;
		
		audio_thread_terminate() ;
		
printf("after term\n") ;
		
	
		clear_spec((GtkWidget*) but) ;
		
			gdk_input_remove(callback_tag) ;
			callback_tag = 0 ;
			printf("set callback tag to 0\n") ;
			close(sd) ;
		
		// clean audio window here
	}
}



void audio_window_accept(gpointer data, gint source,  GdkInputCondition con) {
	struct sockaddr_un cliaddr ;
	int tmp_sd ;
	
	socklen_t clilen = sizeof(cliaddr) ;

	tmp_sd = accept(listen_sd, (struct sockaddr *) &cliaddr, &clilen) ;
	
	if(tmp_sd == -1) {
		printf("error %s\n", strerror(errno) ) ;
		return ;
	}
	

	if(callback_tag) {
		close(tmp_sd) ;
		printf("closed connection to \n") ;
	} else {
		sd = tmp_sd ;
		printf("got connection\n") ;
		callback_tag = gdk_input_add( tmp_sd, GDK_INPUT_READ, audio_window_update, (gpointer) data );
	}

	printf("AW: return from accept\n") ;
	return ;
}


/*
 * Create a new audio window
 *
 *
 */
GtkWidget* audio_window_create (void) {
  GtkWidget *window3;
  GtkWidget *table1;
  GtkWidget *drawingarea1;
  GtkWidget *drawingarea2;
  GtkWidget *table2;
  GtkWidget *label1;
  GtkWidget *label2;
 
  GtkWidget *delay_vscale;
  GtkWidget *sens_vscale;
  GtkWidget *hbox1 ;
  GdkColor color;
  GSList *beat_button_group = NULL;
  GtkWidget *label3 ;
  GtkWidget *checkbutton3;
	
  struct sockaddr_un servaddr ;
		
  window3 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window3), "Audio Input");
  gtk_window_set_position (GTK_WINDOW (window3), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (window3), FALSE);

  table1 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (window3), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 2);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 2);

  hbox1 = gtk_hbox_new(FALSE ,0) ;
  gtk_widget_show (hbox1);
  gtk_table_attach (GTK_TABLE (table1), hbox1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  
  
  label3 = gtk_label_new ("Mode:");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox1), label3, FALSE, FALSE, 0);
  
  beat_button = gtk_radio_button_new_with_mnemonic (NULL, "Beat Triggered");
  gtk_widget_show (beat_button);
  gtk_box_pack_start (GTK_BOX (hbox1), beat_button, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (beat_button), beat_button_group);
  beat_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (beat_button));

  slow_button = gtk_radio_button_new_with_mnemonic (NULL, "Slow");
  gtk_widget_show (slow_button);
  gtk_box_pack_start (GTK_BOX (hbox1), slow_button, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (slow_button), beat_button_group);
  beat_button_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (slow_button));
  
  gtk_toggle_button_set_active((GtkToggleButton*) slow_button, TRUE) ;
  
  checkbutton2 = gtk_check_button_new_with_mnemonic ("Beat");
  gtk_widget_show (checkbutton2);
  gtk_table_attach (GTK_TABLE (table1), checkbutton2, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_button_set_focus_on_click (GTK_BUTTON (checkbutton2), FALSE);
  
  
  drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea1);
  gtk_table_attach (GTK_TABLE (table1), drawingarea1, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (drawingarea1, 256, 128);

  drawingarea2 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea2);
  gtk_table_attach (GTK_TABLE (table1), drawingarea2, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (drawingarea2, 256, 128);

  table2 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table2);
  gtk_table_attach (GTK_TABLE (table1), table2, 0, 1, 1, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);


  label1 = gtk_label_new ("Sens");
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table2), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_CENTER);
  gtk_misc_set_padding (GTK_MISC (label1), 7, 0);

  label2 = gtk_label_new ("Delay");
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table2), label2, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_CENTER);
  gtk_misc_set_padding (GTK_MISC (label2), 7, 0);

  sens_vscale = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (200, 0, 500, 20, 50, 0)));
  gtk_widget_show (sens_vscale);
  gtk_table_attach (GTK_TABLE (table2), sens_vscale, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_scale_set_digits (GTK_SCALE (sens_vscale), 0);
  gtk_range_set_inverted (GTK_RANGE (sens_vscale), TRUE);

  delay_vscale = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (1, 0, 30, 1, 10, 0)));
  gtk_widget_show (delay_vscale);
  gtk_table_attach (GTK_TABLE (table2), delay_vscale, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_scale_set_digits (GTK_SCALE (delay_vscale), 0);
  gtk_range_set_inverted (GTK_RANGE (delay_vscale), TRUE);

					
  checkbutton3 = gtk_check_button_new_with_mnemonic ("Enable Input");
  gtk_widget_show (checkbutton3);
  gtk_table_attach (GTK_TABLE (table2), checkbutton3, 0, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
					
    g_signal_connect ((gpointer) delay_vscale, "value_changed",
                    G_CALLBACK (on_delay_vscale_value_changed),
                    NULL);
	

    g_signal_connect ((gpointer) sens_vscale, "value_changed",
                    G_CALLBACK (on_sens_vscale_value_changed),
                    NULL);
	
    g_signal_connect ((gpointer) checkbutton3, "toggled",
                    G_CALLBACK (on_enable_button_toggled),
                    NULL);
					
	sens = gtk_range_get_value((GtkRange*) sens_vscale) ;
	delay = SAMPLES * gtk_range_get_value((GtkRange*) delay_vscale) ;
	
	
	draw1_pixmap = init_draw_area(drawingarea1) ;
	draw2_pixmap = init_draw_area(drawingarea2) ;

	// new graphics contexts
	gc1 = gdk_gc_new(drawingarea1->window);
	
	bar_pixmap = init_bars( drawingarea1->window) ;

	// set foreground to black
	gdk_color_black(gdk_colormap_get_system(),&color);
	gdk_gc_set_foreground(gc1,&color);
	
	// black drawing pixmaps
	gdk_draw_rectangle (draw1_pixmap, gc1, TRUE, 0, 0, drawingarea1->allocation.width, drawingarea1->allocation.height);
	gdk_draw_rectangle (draw2_pixmap, gc1, TRUE, 0, 0, drawingarea2->allocation.width, drawingarea2->allocation.height);

	// set bg pixmaps
	gdk_window_set_back_pixmap(drawingarea1->window,draw1_pixmap,0);
	gdk_window_set_back_pixmap(drawingarea2->window,draw2_pixmap,0);
	
	// draw
	gdk_window_clear(drawingarea1->window) ;
	gdk_window_clear(drawingarea2->window) ;
  
    // init the scales
	scale = HEIGHT / ( log((1 - d) / d) * 2 );
	x00 = d*d*32768.0/(2 * d - 1);
	y00 = -log(-x00) * scale;
	
  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (window3, window3, "window3");
  GLADE_HOOKUP_OBJECT (window3, table1, "table1");
  GLADE_HOOKUP_OBJECT (window3, drawingarea1, "drawingarea1");
  GLADE_HOOKUP_OBJECT (window3, drawingarea2, "drawingarea2");
  GLADE_HOOKUP_OBJECT (window3, table2, "table2");
  GLADE_HOOKUP_OBJECT (window3, label1, "label1");
  GLADE_HOOKUP_OBJECT (window3, label2, "label2");
  GLADE_HOOKUP_OBJECT (window3, checkbutton2, "checkbutton1");
  GLADE_HOOKUP_OBJECT (window3, sens_vscale, "vscale1");
  GLADE_HOOKUP_OBJECT (window3, delay_vscale, "sens_vscale");
  	
	// listen on socket
  listen_sd = socket(AF_LOCAL, SOCK_STREAM, 0) ;
  
  if(listen_sd == -1) {
	  printf("socket failed\n") ;
  }
  
  unlink(LOCAL_PATH) ;
  
  memset(&servaddr, 0x00, sizeof(servaddr)  );
  servaddr.sun_family = AF_LOCAL ;
  strcpy(servaddr.sun_path, LOCAL_PATH )  ;
  
  if( bind(listen_sd, (struct sockaddr *) &servaddr, sizeof(servaddr) ) == -1 ) {
	  printf("bind failed\n") ;
  }
  
  
  if( listen(listen_sd, 5) == -1 ) {
	  printf("lkisten failed\n") ;
  }

  gdk_input_add( listen_sd , GDK_INPUT_READ, audio_window_accept , (gpointer) window3 );
  return window3;
}






/* 
 * Returns 1 on beat, 0 on no beat_callback
 * contains a debounce so you won't get two beats within BOUNCE samples
 */
int detect_beat(gint16 data[CHANNELS][WIDTH], int threshold) {
	static int last_beat =0;
	static gint16 noise[2][4];
	
	// fall off rate (reduction per sample)
	#define SPEED 20
	
	// don't trigger two beats within BOUNCE samples
	#define BOUNCE 30 
	
	int t, c, tmp;
	int chk1 ;
	
	for (c = 0; c < 2; c++) { 
		for (t = 0; t < 4; t++) {
			tmp = data[c][t]>>7;
			if (tmp > noise[c][t]) noise[c][t] = tmp;
			else if (noise[c][t]> SPEED) noise[c][t] -= SPEED;
			else noise[c][t] = 0; 
		}
	}

	// sum of noise[0] L and R
	chk1 = noise[0][0] + noise[1][0];

	if(last_beat < BOUNCE) 
		last_beat ++ ;
	
	if (chk1 > threshold && last_beat >= BOUNCE ) {
		last_beat = 0 ;	
		return 1;
	}

	return 0;			
}





/*
 * Called when their is action on the input pipe
 * Reads the data, then updates the display
 *
 */
void audio_window_update(gpointer widget, gint source,  GdkInputCondition con) {
	GtkWidget *audioWindow = (GtkWidget*) widget; 
	gint16 data[CHANNELS][WIDTH] ;
	
	// start up in slow mode
	static int mode = SLOW_NOT_SIGNALED ;
	static int time_in_mode = 0 ;
	
	int ret ;
	int event ;
	
	// read from pipe ;	
	ret = read(sd, data, sizeof(data) );

	if(ret != sizeof(data)) {
		
		if( ret == 0) {
			// source has disconencted
			
			gdk_input_remove(callback_tag) ;
			callback_tag = 0 ;
			printf("set callback tag to 0\n") ;
			close(sd) ;
			clear_spec(audioWindow) ;
		}
		
		printf("ret is %d, data is %d\n", ret, sizeof(data)) ;
		return ;
	}
	
	if(GTK_WIDGET_VISIBLE(audioWindow)) {
		// draw spec
		draw_spect(audioWindow, data);
	}

	// returns BEAT, SLOW or AUDIO_NONE
	ret = detect_beat(data, sens) ;
	
	time_in_mode++ ;
	
	if(ret == 1) {
		if(gtk_toggle_button_get_active( (GtkToggleButton*) checkbutton2) ) {
			gtk_toggle_button_set_active( (GtkToggleButton*) checkbutton2 , FALSE ) ;
		} else {
			gtk_toggle_button_set_active( (GtkToggleButton*) checkbutton2 , TRUE ) ;	
		}
	}

	event = AUDIO_NONE ;
	
	if(mode == BEATING) {
		
		// got a beat
		if( ret == AUDIO_BEAT) {
			time_in_mode = 0;
			event = AUDIO_BEAT ;
		}
		
		// if timeout
		if(time_in_mode >= delay) {
			time_in_mode = 0;
			gtk_toggle_button_set_active((GtkToggleButton*) slow_button, TRUE) ;
			// if not callback defined,
			if(beat_callback == NULL) {
				// goto slow not signaled
				mode = SLOW_NOT_SIGNALED ;
			} else {
				// else goto slow mode and do callback
				mode = SLOW ;
				event = AUDIO_SLOW ;
			}
		}

	// slow mode		
	} else if( mode == SLOW) {
		
		if(ret == AUDIO_BEAT) {
			
			// got a beat, move to trans mode
			event = AUDIO_TRANS ;
			mode = TRANSITION ;
			time_in_mode = 0 ;
		} else if(beat_callback == NULL) {
			
			// a valid callback  moves us to slow mode
			mode = SLOW_NOT_SIGNALED;
			time_in_mode = 0 ;
		}
		
	} else if (mode == SLOW_NOT_SIGNALED) {

		if(ret == AUDIO_BEAT) {
			// another beat, move to beat mode
			event = AUDIO_TRANS;
			mode = TRANSITION ;
			time_in_mode = 0 ;
		} else if(beat_callback != NULL) {
			
			// a valid callback  moves us to slow mode
			mode = SLOW ;
			time_in_mode = 0 ;
			event = AUDIO_SLOW ;
		}
		
		
	} else if (mode == TRANSITION) {
		

		if(ret == AUDIO_BEAT) {
			// a second beat moves to beat mode
			mode = BEATING ;
			time_in_mode = 0;
			gtk_toggle_button_set_active((GtkToggleButton*) beat_button, TRUE);
			event = AUDIO_BEAT ;
			
		} else if(time_in_mode >= delay) {
			// if timeout
			time_in_mode = 0;
			
			// if not callback defined,
			if(beat_callback == NULL) {
				// goto state slow not signaled
				mode = SLOW_NOT_SIGNALED ;
			} else {
				// else goto slow mode
				mode = SLOW ;
				event = AUDIO_SLOW ;
			}
		}
	}
	
	if( beat_callback != NULL) {
		beat_callback(event, callback_data) ;
	}
	
}



int audio_window_set_callback(void (*fh)(audio_events event,void *data), void *data) {
	beat_callback = fh ;
	callback_data = data ;	
	return 1;
}
